/*
 * Copyright (C) 2013-2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UBUNTU_APPLICATION_PLUGIN_LOGGING_H
#define UBUNTU_APPLICATION_PLUGIN_LOGGING_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(QTMIR_APPLICATIONS)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_SESSIONS)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_SURFACES)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_MIR_MESSAGES)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_SENSOR_MESSAGES)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_MIR_INPUT)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_MIR_KEYMAP)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_CLIPBOARD)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_SCREENS)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_DBUS)
Q_DECLARE_LOGGING_CATEGORY(QTMIR_URLBUILDER)

#endif // UBUNTU_APPLICATION_PLUGIN_LOGGING_H
