/*
 * Copyright (C) 2015 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mock_settings.h"

namespace qtmir
{

MockSettings::MockSettings()
{
    using namespace ::testing;

    QVariantList lifecycleExemptAppIds;
    lifecycleExemptAppIds << "music.ubports";
    ON_CALL(*this, get(_))
            .WillByDefault(
                Return(lifecycleExemptAppIds));

}

MockSettings::~MockSettings()
{

}

} // namespace qtmir

