/*
 * Copyright (C) 2016,2017 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "logging.h"

Q_LOGGING_CATEGORY(QTMIR_APPLICATIONS, "qtmir.applications", QtInfoMsg)
Q_LOGGING_CATEGORY(QTMIR_SESSIONS, "qtmir.sessions")
Q_LOGGING_CATEGORY(QTMIR_SURFACES, "qtmir.surfaces", QtInfoMsg)
Q_LOGGING_CATEGORY(QTMIR_MIR_INPUT, "qtmir.mir.input", QtWarningMsg)
Q_LOGGING_CATEGORY(QTMIR_MIR_MESSAGES, "qtmir.mir")
Q_LOGGING_CATEGORY(QTMIR_MIR_KEYMAP, "qtmir.mir.keymap")
Q_LOGGING_CATEGORY(QTMIR_CLIPBOARD, "qtmir.clipboard")
Q_LOGGING_CATEGORY(QTMIR_SENSOR_MESSAGES, "qtmir.sensor")
Q_LOGGING_CATEGORY(QTMIR_SCREENS, "qtmir.screens")
Q_LOGGING_CATEGORY(QTMIR_DBUS, "qtmir.dbus", QtWarningMsg)
Q_LOGGING_CATEGORY(QTMIR_URLBUILDER, "qtmir.urlbuilder", QtInfoMsg)
